__author__ = 'mnowotka'

from django.views.generic import TemplateView

#-----------------------------------------------------------------------------------------------------------------------

class DirectTemplateView(TemplateView):
    extra_context = None
    def get_context_data(self, **kwargs):
        context = super(self.__class__, self).get_context_data(**kwargs)
        if self.extra_context is not None:
            for key, value in self.extra_context.items():
                if callable(value):
                    context[key] = value()
                else:
                    context[key] = value
        return context

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from importlib import import_module
from django.conf.urls import patterns, url
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from tastypie_spore_docs.direct_template_view import DirectTemplateView
from tastypie_spore_docs.views import generate_spore_endpoint
try:
    from django.utils.module_loading import import_string
except ImportError:
    from django.utils.module_loading import import_by_path as import_string

try:
    api = import_string(settings.TASTYPIE_DOC_API)
except AttributeError:
    raise ImproperlyConfigured('TASTYPIE_DOC_API setting is required')


try:
    name = settings.TASTYPIE_DOC_NAME
except AttributeError:
    name = 'Tastypie REST API documentation'

api_name = api.api_name
api_module = import_module('.'.join(settings.TASTYPIE_DOC_API.split('.')[:-1]))
try:
    version = api_module.__version__
except AttributeError:
    version = api_name

urlpatterns = patterns('',
    url(r'^%s/docs' % api_name, DirectTemplateView.as_view(template_name="docs.html"), name='ws_docs'),
    url(r'^%s/spore' % api_name, generate_spore_endpoint, {'api': api, 'name': name, 'version':version}, name='ws_spore_endpoint'),
)



__author__ = 'mnowotka'

from django.http import HttpResponse
from django.core.urlresolvers import reverse, NoReverseMatch
from django.test import RequestFactory
from collections import OrderedDict
import json

#-----------------------------------------------------------------------------------------------------------------------

def generate_spore_endpoint(request, api, name, version):

    envelope = {
        "expected_status": [200],
        "version": version,
        "name": name,
        "methods": OrderedDict()
    }

    registry = api._registry
    factory = RequestFactory()

    for resource_name, resource in sorted(registry.items()):
        #print 'analysing ' + resource_name + ' resource'
        try:
            schema_url = reverse('api_get_schema', kwargs={'api_name':api.api_name, 'resource_name': resource_name})
        except NoReverseMatch:
            print 'No schema for ' + resource_name + ' resource'
            continue
        formats = resource._meta.serializer.formats
        required_params_dict = None
        if hasattr(resource._meta, 'required_params'):
            required_params_dict = resource._meta.required_params
            print required_params_dict
        request = factory.get(schema_url)
        request.format = 'json'
        schema = resource.get_schema(request)
        schema_dict = json.loads(schema.content)
        allowed_detail_http_methods = schema_dict['allowed_detail_http_methods']
        allowed_list_http_methods = schema_dict['allowed_list_http_methods']
        available_methods = [url.name for url in resource.urls]

        #print 'available_methods:'
        #print available_methods

        for verb in ['get', 'post']:
            for type in ['api_dispatch_list', 'api_dispatch_detail', 'api_get_multiple']:
                if type not in available_methods:
                    continue
                if type in ('api_dispatch_list','api_get_multiple') and verb not in allowed_list_http_methods:
                    continue
                elif type == 'api_dispatch_detail' and verb not in allowed_detail_http_methods:
                    continue
                method_name = verb.upper() + '_' + resource_name + '_' + type
                canonical_url = schema_url[:schema_url.find('/schema') + 1]
                if type == 'api_dispatch_list':
                    description = "Retrieve " + resource_name + " object list."
                    required_params = []
                elif type == 'api_dispatch_detail':
                    canonical_url += ':ID'
                    description = "Retrieve single " + resource_name + " object details by ID."
                    required_params = ['ID']
                elif type == 'api_get_multiple':
                    canonical_url += 'set/:IDs_list'
                    description = "Retrieve multiple " + resource_name + " objects by IDs."
                    required_params = ['IDs_list']
                if required_params_dict:
                    required_params = required_params_dict.get(type, required_params)
                    canonical_url = canonical_url[:canonical_url.find(':')] + '/'.join([':' + x for x in required_params])
                method_data = {
                    "method" : verb.upper(),
                    "description": description,
                    "path" : canonical_url,
                    "required_params": required_params,
                    "formats": formats
                }
                envelope["methods"][method_name] = method_data

    return HttpResponse(json.dumps(envelope), content_type="application/json")

#-----------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'mnowotka'

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

setup(
    name='tastypie_spore_docs',
    version='0.1.2',
    author='Michal Nowotka',
    author_email='mnowotka@ebi.ac.uk',
    description='This Django app generates SPORE endpoint from Tastypie REST API and provides a JavaScript based SPORE client.',
    url='https://www.ebi.ac.uk/chembldb/index.php/ws',
    license='Apache Software License',
    packages=['tastypie_spore_docs'],
    long_description=open('README.rst').read(),
    install_requires=[
        'django-tastypie',
    ],
    include_package_data=True,
    classifiers=['Development Status :: 2 - Pre-Alpha',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Software Development :: Documentation'],
    zip_safe=False,
)
